"""
Created on 13/mar/2014

@author: Marco Pompili
"""

from django.db import models
from django.utils.translation import ugettext_lazy as _


class InstagramConfiguration(models.Model):
    class Meta:
        verbose_name = _(u"Django Instagram Configuration")
        verbose_name_plural = _(u"Django Instagram Configuration")

    app_access_token = models.TextField(
        _(u'Access token'),
        help_text=_(
            u'Click on "Get Access Token" and you will be redirected to \
            Instagram, then follow the instructions. Remember to log in with \
            the account that you want to show on the web page.'))



"""
Created on 24/gen/2014

@author: Marco Pompili
"""

import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name=str('django-instagram'),
    packages=['django_instagram', 'django_instagram.templatetags'],
    version=str('0.1.1'),
    author=str('Marco Pompili'),
    author_email=str('marcs.pompili@gmail.org'),
    url=str('https://github.com/marcopompili/django-instagram'),
    description=str('Instagram client for Django.'),
    long_description=README,
    include_package_data=True,
    license=str('BSD-3 License'),
    install_requires=[
        'django>=1.6',
        'python-instagram'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



"""
Created on 12/dic/2013

@author: Marco Pompili
"""

from instagram.client import InstagramAPI

from django import template

from django_instagram.models import InstagramConfiguration

register = template.Library()

DJANGO_INSTAGRAM_CLIENT_ID = "5668831cb13b4876a63fa53fe768927e"
DJANGO_INSTAGRAM_REDIRECT_URI = "http://www.emarcs.net/instagram/"


def instagram_sign_in_with_token():
    token = InstagramConfiguration.objects.first()

    if token is None:
        print("Django Instagram, configuration not found")
        return token
    else:
        return InstagramAPI(access_token=token.app_access_token)


def instagram_get_recent_media(count):
    api = instagram_sign_in_with_token()

    if api:
        recent_media, next_ = api.user_recent_media(count=count)
        return recent_media
    else:
        return None


class InstagramPopularMediaNode(template.Node):
    """
        Template node for displaying the popular media of Instagram.
        The API doesn't need authentication, just basic access.
    """
    def __init__(self, count):
        self.count = count
        self.api = InstagramAPI(client_id=DJANGO_INSTAGRAM_CLIENT_ID)

    def render(self, context):
        popular_media = self.api.media_popular(self.count)

        context['popular_media'] = popular_media

        return ''


@register.tag
def instagram_popular_media(parser, token):
    """
        Tag for getting data about popular media on Instagram.
    """
    try:
        tagname, count = token.split_contents()

        return InstagramPopularMediaNode(count.split('=')[1])
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires a single argument" % token.contents.split()[0]
        )


class InstagramRecentMediaNode(template.Node):
    """
        Template node for showing recent media from an user.
        This functionality needs an access token.
        The access token should be set in the administration interface.

        If no access token is set the process should fail silently,
        so watch out for error messages in the console.
    """

    def __init__(self, count):
        self.count = count

    def render(self, context):
        context['recent_media'] = instagram_get_recent_media(self.count)

        return ''


@register.tag
def instagram_recent_media(parser, token):
    """
        Tag for getting data about recent media of an user.
    """
    try:
        tagname, count = token.split_contents()

        return InstagramRecentMediaNode(count.split('=')[1])
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires a single argument" % token.contents.split()[0]
        )


@register.inclusion_tag('django_instagram/recent_media_box.html')
def instagram_recent_media_box(*args, **kwargs):
    recent_media = instagram_get_recent_media(count=kwargs.get('count', 10))

    return {
        'recent_media': recent_media
    }


@register.inclusion_tag('django_instagram/recent_media_wall.html')
def instagram_recent_media_wall(*args, **kwargs):
    recent_media = instagram_get_recent_media(count=kwargs.get('count', 10))

    return {
        'recent_media': recent_media
    }

@register.filter(name='standard_size')
def instagram_standard_size(value):
    return value.images['standard_resolution'].url


@register.filter(name='low_resolution')
def instagram_low_resolution(value):
    return value.images['low_resolution'].url


@register.filter(name='thumbnail')
def instagram_thumbnail(value):
    return value.images['thumbnail'].url



"""
Created on 15/dic/2013

@author: Marco Pompili
"""

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from django.db import transaction
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_protect

from .models import InstagramConfiguration
from .templatetags import instagram_client


csrf_protect_m = method_decorator(csrf_protect)

instagram_extra_content = {
    "client_id": instagram_client.DJANGO_INSTAGRAM_CLIENT_ID,
    "redirect_uri": instagram_client.DJANGO_INSTAGRAM_REDIRECT_URI
}


class InstagramAdmin(admin.ModelAdmin):
    fieldsets = (
        (_(u'Django Instagram Configuration'),
         {
             'fields': ('app_access_token',)
         }),
    )

    def has_add_permission(self, request):
        """
        Just let to add the configuration once.
        :param request:
        :return:
        """
        return not InstagramConfiguration.objects.exists()

    def add_view(self, request, form_url='', extra_context=None):
        """
        Add extra parameters from the settings (client_id, redirect_uri) into the add view.
        :param request:
        :param form_url:
        :param extra_context:
        :return:
        """
        return admin.ModelAdmin.add_view(self,
                                         request,
                                         form_url='',
                                         extra_context=instagram_extra_content)

    @csrf_protect_m
    @transaction.atomic
    def change_view(self, request, object_id, form_url='', extra_context=None):
        """
        Add extra parameters from the settings (client_id, redirect_uri) into the change_view.
        :param request:
        :param object_id:
        :param form_url:
        :param extra_context:
        :return:
        """
        return admin.ModelAdmin.change_view(self,
                                            request,
                                            object_id,
                                            form_url=form_url,
                                            extra_context=instagram_extra_content)


admin.site.register(InstagramConfiguration, InstagramAdmin)